"use strict";
var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __defNormalProp = (obj, key, value) => key in obj ? __defProp(obj, key, { enumerable: true, configurable: true, writable: true, value }) : obj[key] = value;
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __copyProps = (to, from, except, desc) => {
  if (from && typeof from === "object" || typeof from === "function") {
    for (let key of __getOwnPropNames(from))
      if (!__hasOwnProp.call(to, key) && key !== except)
        __defProp(to, key, { get: () => from[key], enumerable: !(desc = __getOwnPropDesc(from, key)) || desc.enumerable });
  }
  return to;
};
var __toCommonJS = (mod) => __copyProps(__defProp({}, "__esModule", { value: true }), mod);
var __publicField = (obj, key, value) => {
  __defNormalProp(obj, typeof key !== "symbol" ? key + "" : key, value);
  return value;
};
var config_exports = {};
__export(config_exports, {
  ConfigController: () => ConfigController,
  configController: () => configController
});
module.exports = __toCommonJS(config_exports);
var import_utils = require("../../utils");
class ConfigController {
  /**
   * @param options  The options to construct the controller.
   */
  constructor({ inject }) {
    /**
     * The service in charge or building the responses.
     */
    __publicField(this, "_responsesBuilder");
    /**
     * The service in charge of the configuration.
     */
    __publicField(this, "_config");
    this._responsesBuilder = inject.responsesBuilder;
    this._config = inject.config;
  }
  /**
   * Creates the middleware the shows the current configuration.
   */
  showConfig() {
    return (_, res) => {
      this.respondWithConfig(res);
    };
  }
  /**
   * Creates the middleware the allows to switch the configuration.
   */
  switchConfig() {
    return async (req, res, next) => {
      const { name } = req.params;
      if (!name || !this._config.canSwitchConfigs()) {
        next();
        return;
      }
      try {
        await this._config.switch(name);
        this.respondWithConfig(res);
      } catch (error) {
        next(error);
      }
    };
  }
  /**
   * Utility to respond with the current configuration.
   */
  respondWithConfig(res) {
    const name = this._config.get("name");
    const data = {
      name,
      ...this._config.getConfig()
    };
    this._responsesBuilder.json({
      res,
      data
    });
  }
}
const configController = (0, import_utils.controller)((app) => {
  const config = app.getConfig();
  const router = app.getRouter();
  if (config.get("debug.configController") !== true) {
    return router;
  }
  const ctrl = new ConfigController({
    inject: {
      config,
      responsesBuilder: app.get("responsesBuilder")
    }
  });
  return router.get("/", ctrl.showConfig()).get("/switch/:name", ctrl.switchConfig());
});
// Annotate the CommonJS export names for ESM import in node:
0 && (module.exports = {
  ConfigController,
  configController
});
//# sourceMappingURL=config.js.map