import * as _homer0_jimple from '@homer0/jimple';
import { Statuses } from '../../utils/fns/statuses.mjs';
import { r as JimpexHealthStatus, C as ControllerConnectFn } from '../../index-Bwf7JHu9.mjs';
import { AsyncExpressMiddleware } from '../../types/express.mjs';
import { SimpleConfig } from '@homer0/simple-config';
import { ResponsesBuilder } from '../../services/http/responsesBuilder.mjs';
import 'statuses';
import '../../types/http.mjs';
import 'https';
import 'http';
import 'spdy';
import 'node-fetch';
import 'express';
import '../../types/utils.mjs';
import '@homer0/path-utils';
import '@homer0/simple-logger';
import '@homer0/events-hub';

/**
 * A function that will return the health status of the application.
 *
 * @group Controllers/Health
 */
type GetHealthStatus = () => Promise<JimpexHealthStatus>;
/**
 * The options to contruct a {@link HealthController}.
 *
 * @group Controllers/Health
 */
type HealthControllerOptions = {
    /**
     * A dictionary with the dependencies to inject.
     */
    inject: {
        getHealthStatus: GetHealthStatus;
        responsesBuilder: ResponsesBuilder;
        config: SimpleConfig;
        statuses: Statuses;
    };
};
/**
 * The controller class that shows the application health status.
 *
 * @group Controller Classes
 * @group Controllers/Health
 * @prettierignore
 */
declare class HealthController {
    /**
     * The function that returns the health status of the application.
     */
    protected readonly _getHealthStatus: GetHealthStatus;
    /**
     * The service in charge or building the responses.
     */
    protected readonly _responsesBuilder: ResponsesBuilder;
    /**
     * The service in charge of the configuration.
     */
    protected readonly _config: SimpleConfig;
    /**
     * The uility service to get HTTP status codes.
     */
    protected readonly _statuses: Statuses;
    /**
     * @param options  The options to construct the controller.
     */
    constructor({ inject }: HealthControllerOptions);
    /**
     * Creates the middleware the shows the application health status.
     */
    showHealth(): AsyncExpressMiddleware;
}
/**
 * The controller that once mounted, it will add an endpoint to show the application
 * health status.
 *
 * @group Controllers
 * @group Controllers/Health
 */
declare const healthController: _homer0_jimple.Resource<"controller", "connect", ControllerConnectFn>;

export { type GetHealthStatus, HealthController, type HealthControllerOptions, healthController };
