"use strict";
var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __defNormalProp = (obj, key, value) => key in obj ? __defProp(obj, key, { enumerable: true, configurable: true, writable: true, value }) : obj[key] = value;
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __copyProps = (to, from, except, desc) => {
  if (from && typeof from === "object" || typeof from === "function") {
    for (let key of __getOwnPropNames(from))
      if (!__hasOwnProp.call(to, key) && key !== except)
        __defProp(to, key, { get: () => from[key], enumerable: !(desc = __getOwnPropDesc(from, key)) || desc.enumerable });
  }
  return to;
};
var __toCommonJS = (mod) => __copyProps(__defProp({}, "__esModule", { value: true }), mod);
var __publicField = (obj, key, value) => {
  __defNormalProp(obj, typeof key !== "symbol" ? key + "" : key, value);
  return value;
};
var health_exports = {};
__export(health_exports, {
  HealthController: () => HealthController,
  healthController: () => healthController
});
module.exports = __toCommonJS(health_exports);
var import_utils = require("../../utils");
class HealthController {
  /**
   * @param options  The options to construct the controller.
   */
  constructor({ inject }) {
    /**
     * The function that returns the health status of the application.
     */
    __publicField(this, "_getHealthStatus");
    /**
     * The service in charge or building the responses.
     */
    __publicField(this, "_responsesBuilder");
    /**
     * The service in charge of the configuration.
     */
    __publicField(this, "_config");
    /**
     * The uility service to get HTTP status codes.
     */
    __publicField(this, "_statuses");
    this._getHealthStatus = inject.getHealthStatus;
    this._responsesBuilder = inject.responsesBuilder;
    this._config = inject.config;
    this._statuses = inject.statuses;
  }
  /**
   * Creates the middleware the shows the application health status.
   */
  showHealth() {
    return async (_, res) => {
      const healthStatus = await this._getHealthStatus();
      let isHealthy;
      let extras;
      if (typeof healthStatus === "boolean") {
        isHealthy = healthStatus;
        extras = {};
      } else {
        if (typeof healthStatus.isHealthy === "boolean") {
          isHealthy = healthStatus.isHealthy;
        } else if (healthStatus.services) {
          isHealthy = Object.values(healthStatus.services).every(
            (value) => value === true
          );
        } else {
          isHealthy = true;
        }
        extras = {
          ...healthStatus.services
        };
      }
      const { name: config, version } = this._config.get(["name", "version"]);
      const status = isHealthy ? this._statuses("ok") : this._statuses("service unavailable");
      this._responsesBuilder.json({
        res,
        status,
        data: {
          isHealthy,
          status,
          config,
          version,
          ...extras
        }
      });
    };
  }
}
const healthController = (0, import_utils.controller)((app) => {
  const router = app.getRouter();
  const ctrl = new HealthController({
    inject: {
      getHealthStatus: app.isHealthy.bind(app),
      responsesBuilder: app.get("responsesBuilder"),
      config: app.get("config"),
      statuses: app.get("statuses")
    }
  });
  return router.get("/", ctrl.showHealth());
});
// Annotate the CommonJS export names for ESM import in node:
0 && (module.exports = {
  HealthController,
  healthController
});
//# sourceMappingURL=health.js.map