"use strict";
var __create = Object.create;
var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __getProtoOf = Object.getPrototypeOf;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __defNormalProp = (obj, key, value) => key in obj ? __defProp(obj, key, { enumerable: true, configurable: true, writable: true, value }) : obj[key] = value;
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __copyProps = (to, from, except, desc) => {
  if (from && typeof from === "object" || typeof from === "function") {
    for (let key of __getOwnPropNames(from))
      if (!__hasOwnProp.call(to, key) && key !== except)
        __defProp(to, key, { get: () => from[key], enumerable: !(desc = __getOwnPropDesc(from, key)) || desc.enumerable });
  }
  return to;
};
var __toESM = (mod, isNodeMode, target) => (target = mod != null ? __create(__getProtoOf(mod)) : {}, __copyProps(
  // If the importer is in node compatibility mode or this is not an ESM
  // file that has been converted to a CommonJS file using a Babel-
  // compatible transform (i.e. "__esModule" has not been set), then set
  // "default" to the CommonJS "module.exports" for node compatibility.
  isNodeMode || !mod || !mod.__esModule ? __defProp(target, "default", { value: mod, enumerable: true }) : target,
  mod
));
var __toCommonJS = (mod) => __copyProps(__defProp({}, "__esModule", { value: true }), mod);
var __publicField = (obj, key, value) => {
  __defNormalProp(obj, typeof key !== "symbol" ? key + "" : key, value);
  return value;
};
var statics_exports = {};
__export(statics_exports, {
  StaticsController: () => StaticsController,
  staticsController: () => staticsController
});
module.exports = __toCommonJS(statics_exports);
var path = __toESM(require("path"));
var import_deep_assign = require("@homer0/deep-assign");
var import_utils = require("../../utils");
class StaticsController {
  /**
   * @param options  The options to construct the controller.
   */
  constructor({ inject, ...options }) {
    /**
     * The service that serves static files.
     */
    __publicField(this, "_sendFile");
    /**
     * The MIME type library. Since it's an ESM only module, Jimpex loads it on boot and makes
     * it available on the container.
     */
    __publicField(this, "_mime");
    /**
     * The controller customization options.
     */
    __publicField(this, "_options");
    /**
     * A dictionary with the formatted definitions of the files that will be served.
     * It uses the files' routes as keys, for easy access in the middleware.
     */
    __publicField(this, "files");
    this._sendFile = inject.sendFile;
    this._mime = inject.mime;
    this._options = this._validateOptions(
      (0, import_deep_assign.deepAssignWithOverwrite)(
        {
          files: ["favicon.ico", "index.html"],
          methods: options.methods || {
            all: false,
            get: true
          },
          paths: {
            route: "",
            source: "./"
          }
        },
        options
      )
    );
    this.files = this._createFiles();
  }
  /**
   * Mounts the middlewares in the router in order to serve the files.
   *
   * @param router       A reference to the application router.
   * @param middlewares  A list of extra middlewares to execute before the file
   *                     middleware.
   */
  addRoutes(router, middlewares = []) {
    const { methods } = this._options;
    const use = methods.all ? ["all"] : Object.keys(methods).reduce((acc, name) => {
      const methodName = name;
      if (methods[methodName]) {
        acc.push(methodName);
      }
      return acc;
    }, []);
    Object.keys(this.files).forEach((route) => {
      const file = this.files[route];
      const fileMiddleware = this._getMiddleware(file);
      use.forEach(
        (method) => this._addRoute({ router, method, file, fileMiddleware, middlewares })
      );
    });
    return router;
  }
  /**
   * The controller options.
   */
  get options() {
    return { ...this._options };
  }
  /**
   * Generates the middleware that will serve the file.
   *
   * @param file  The definition of the file to serve.
   */
  _getMiddleware(file) {
    return (_, res, next) => {
      const extension = path.parse(file.path).ext.substring(1);
      const headers = {
        "Content-Type": this._mime.getType(extension) || "text/html",
        ...file.headers
      };
      Object.entries(headers).forEach(([key, value]) => {
        res.setHeader(key, value);
      });
      this._sendFile({
        res,
        filepath: file.path,
        next
      });
    };
  }
  /**
   * Mounts the middleware(s) for a file in the router.
   *
   * @param options  The information of the file and how it needs to be added.
   */
  _addRoute({
    router,
    method,
    file,
    fileMiddleware,
    middlewares
  }) {
    const { route } = file;
    router[method](route, [...middlewares, fileMiddleware]);
  }
  /**
   * Validates and formats the options sent to the constructor in order to get the final
   * set that will be stored in the controller.
   *
   * @param options  The options to validate.
   * @throws If no files are specified.
   * @throws If methods is not defined.
   * @throws If no methods are enabled.
   * @throws If there's an invalid HTTP method.
   */
  _validateOptions(options) {
    if (!options.files || !options.files.length) {
      throw new Error("You need to specify a list of files");
    }
    if (!options.methods) {
      throw new Error("You need to specify which HTTP methods are allowed for the files");
    }
    const methods = Object.keys(options.methods);
    const atLeastOne = methods.some((method) => options.methods[method]);
    if (!atLeastOne) {
      throw new Error("You need to enable at least one HTTP method to serve the files");
    }
    const allowedMethods = [
      "all",
      "get",
      "head",
      "post",
      "patch",
      "put",
      "delete",
      "connect",
      "options",
      "trace"
    ];
    const invalid = methods.find(
      (method) => !allowedMethods.includes(method.toLowerCase())
    );
    if (invalid) {
      throw new Error(`${invalid} is not a valid HTTP method`);
    }
    const newMethods = methods.reduce((acc, method) => {
      acc[method.toLowerCase()] = !!options.methods[method];
      return acc;
    }, {});
    return {
      ...options,
      methods: newMethods
    };
  }
  /**
   * Parses the files received from the constructor's options, and formats them into
   * proper definitions the controller can use.
   */
  _createFiles() {
    const { files, paths } = this._options;
    const routePath = (0, import_utils.removeSlashes)(paths.route, false, true);
    return files.reduce((acc, file) => {
      let src;
      let route;
      let headers;
      if (typeof file === "object") {
        ({ route, path: src, headers } = file);
      } else {
        src = file;
        route = file;
      }
      src = path.join(paths.source, src);
      route = (0, import_utils.removeSlashes)(route, true, false);
      route = `${routePath}/${route}`;
      acc[route] = {
        path: src,
        route,
        headers: headers || {}
      };
      return acc;
    }, {});
  }
}
const staticsController = (0, import_utils.controllerCreator)(
  ({ getMiddlewares, ...options } = {}) => (app) => {
    const router = app.getRouter();
    const ctrl = new StaticsController({
      inject: {
        sendFile: app.get("sendFile"),
        mime: app.get("mime")
      },
      ...options
    });
    let useMiddlewares;
    if (getMiddlewares) {
      useMiddlewares = getMiddlewares(app).map((middleware) => {
        if ("middleware" in middleware) {
          return middleware.connect(app);
        }
        return middleware;
      }).filter(import_utils.notUndefined);
    }
    return ctrl.addRoutes(router, useMiddlewares);
  }
);
// Annotate the CommonJS export names for ESM import in node:
0 && (module.exports = {
  StaticsController,
  staticsController
});
//# sourceMappingURL=statics.js.map