import {
  __publicField
} from "../../chunk-F3FYYIAV.js";
import { middlewareCreator } from "../../utils";
class ForceHTTPS {
  /**
   * @param options  The options to construct the class.
   */
  constructor(options = {}) {
    /**
     * The customization options.
     */
    __publicField(this, "_options");
    this._options = {
      ignoredRoutes: [/^\/service\//],
      ...options
    };
  }
  /**
   * Generates the middleware that redirects the traffic.
   */
  getMiddleware() {
    return (req, res, next) => {
      if (!req.secure && req.get("X-Forwarded-Proto") !== "https" && !this._options.ignoredRoutes.some(
        (expression) => expression.test(req.originalUrl)
      )) {
        const host = req.get("Host");
        res.redirect(`https://${host}${req.url}`);
      } else {
        next();
      }
    };
  }
  /**
   * The customization options.
   */
  get options() {
    return { ...this._options };
  }
}
const forceHTTPSMiddleware = middlewareCreator(
  (options = {}) => (app) => {
    const enabled = app.getConfig("forceHTTPS");
    if (!enabled)
      return void 0;
    return new ForceHTTPS(options).getMiddleware();
  }
);
export {
  ForceHTTPS,
  forceHTTPSMiddleware
};
//# sourceMappingURL=forceHTTPS.js.map