import {
  __publicField
} from "../../chunk-F3FYYIAV.js";
import { middlewareCreator } from "../../utils";
class HSTS {
  /**
   * @param options  The options to construct the class.
   */
  constructor(options = {}) {
    /**
     * The customization options for the header.
     */
    __publicField(this, "_options");
    /**
     * The value of the header that will be included in the responses.
     */
    __publicField(this, "_header");
    this._options = {
      maxAge: 31536e3,
      includeSubDomains: true,
      preload: false,
      ...options
    };
    this._header = this._buildHeader();
  }
  /**
   * Generates the middleware that includes the HSTS header on the responses.
   */
  getMiddleware() {
    return (_, res, next) => {
      res.setHeader("Strict-Transport-Security", this._header);
      next();
    };
  }
  /**
   * The customization options.
   */
  get options() {
    return { ...this._options };
  }
  /**
   * The value of the header that will be included in the responses.
   */
  get header() {
    return this._header;
  }
  /**
   * Creates the header value based on the customization options.
   */
  _buildHeader() {
    const { maxAge, includeSubDomains, preload } = this._options;
    const directives = [`max-age=${maxAge}`];
    if (includeSubDomains) {
      directives.push("includeSubDomains");
    }
    if (preload) {
      directives.push("preload");
    }
    return directives.join("; ");
  }
}
const hstsMiddleware = middlewareCreator(
  (options = {}) => (app) => {
    const setting = app.getConfig("hsts");
    if (!setting?.enabled || typeof options.enabled !== "undefined" && !options.enabled)
      return void 0;
    return new HSTS(options).getMiddleware();
  }
);
export {
  HSTS,
  hstsMiddleware
};
//# sourceMappingURL=hsts.js.map