import "../../chunk-F3FYYIAV.js";
import { provider, statuses as statusesFn } from "../../utils";
import { AppError } from "./appError";
class HTTPError extends AppError {
  /**
   * @param message   The error message.
   * @param status    The HTTP status code.
   * @param context   Context information related to the error.
   * @param statuses  A reference to the service that generates HTTP status codes. This
   *                  is in case the implementation wants to use a special version from
   *                  the container; otherwise, it will use the `statuses` library
   *                  directly.
   */
  constructor(message, status = statusesFn("ok"), context = {}, statuses = statusesFn) {
    super(message, { ...context, status }, statuses);
  }
}
const createHTTPError = (...args) => new HTTPError(...args);
const httpErrorProvider = provider((app) => {
  app.set("HTTPError", () => HTTPError);
  app.set("httpError", () => createHTTPError);
});
export {
  HTTPError,
  createHTTPError,
  httpErrorProvider
};
//# sourceMappingURL=httpError.js.map