import {
  __publicField
} from "../../chunk-F3FYYIAV.js";
import fs from "fs/promises";
import { provider } from "../../utils";
class FrontendFs {
  /**
   * @param options  The options to construct the service.
   */
  constructor({ inject: { pathUtils } }) {
    /**
     * The service that generates the relative paths.
     */
    __publicField(this, "_pathUtils");
    this._pathUtils = pathUtils;
  }
  /**
   * Deletes a file from the file system.
   *
   * @param filepath  The path to the file.
   */
  delete(filepath) {
    return fs.unlink(this.getAppPath(filepath));
  }
  /**
   * Reads a file from the file system.
   *
   * @param filepath  The path to the file.
   * @param encoding  The text encoding in which the file should be read.
   */
  read(filepath, encoding = "utf-8") {
    return fs.readFile(this.getAppPath(filepath), encoding);
  }
  /**
   * Writes a file on the file system.
   *
   * @param filepath  The path to the file.
   * @param content   The contents of the file.
   */
  write(filepath, content) {
    return fs.writeFile(this.getAppPath(filepath), content);
  }
  /**
   * Utility method to get the path of a file relative to the application executable.
   *
   * @param filepath  The path to the file.
   */
  getAppPath(filepath) {
    return this._pathUtils.joinFrom("app", filepath);
  }
}
const frontendFsProvider = provider((app) => {
  app.set(
    "frontendFs",
    () => new FrontendFs({
      inject: {
        pathUtils: app.get("pathUtils")
      }
    })
  );
});
export {
  FrontendFs,
  frontendFsProvider
};
//# sourceMappingURL=frontendFs.js.map