import {
  __publicField
} from "../../chunk-F3FYYIAV.js";
import {
  APIClient as APIClientBase
} from "@homer0/api-utils";
import { deepAssignWithOverwrite } from "@homer0/deep-assign";
import { providerCreator } from "../../utils";
class APIClient extends APIClientBase {
  /**
   * @param options  The options to construct the class.
   */
  constructor({ inject: { http, HTTPError }, ...rest }) {
    const { endpoints, gateway, url } = rest;
    const useEndpoints = endpoints || gateway;
    const apiConfig = {
      url,
      endpoints: useEndpoints
    };
    super({
      ...apiConfig,
      fetchClient: http.fetch
    });
    /**
     * The service that makes the requests to the API.
     */
    __publicField(this, "_http");
    /**
     * The class to generate possible errors in the requests.
     */
    __publicField(this, "_HTTPError");
    /**
     * The configuration of the API it uses.
     */
    __publicField(this, "_apiConfig");
    this._http = http;
    this._HTTPError = HTTPError;
    this._apiConfig = deepAssignWithOverwrite({}, apiConfig);
  }
  /**
   * The configuration for the API.
   */
  get apiConfig() {
    return deepAssignWithOverwrite({}, this._apiConfig);
  }
  /**
   * Tries to obtain a message from an error caused on a failed request.
   *
   * @param response  The response from the failed request.
   */
  _getErrorMessageFromResponse(response) {
    const res = response;
    if (res.error)
      return res.error;
    if (res.data) {
      if (res.data.message)
        return res.data.message;
      if (res.data.error)
        return res.data.error;
    }
    return "Unexpected error";
  }
  /**
   * Generates an {@link _HTTPError} from the response of a failed request.
   *
   * @param response  The response from the failed request.
   * @param status    The status code of the response.
   */
  formatError(response, status) {
    return new this._HTTPError(this._getErrorMessageFromResponse(response), status);
  }
}
const apiClientProvider = providerCreator(
  (options = {}) => (app) => {
    const defaultName = "apiClient";
    const { serviceName = defaultName, clientClass: ClientClass = APIClient } = options;
    let { configSetting } = options;
    if (!configSetting) {
      configSetting = serviceName === defaultName ? "api" : serviceName;
    }
    app.set(
      serviceName,
      () => new ClientClass({
        inject: {
          http: app.get("http"),
          HTTPError: app.get("HTTPError")
        },
        ...app.getConfig(configSetting)
      })
    );
  }
);
export {
  APIClient,
  apiClientProvider
};
//# sourceMappingURL=apiClient.js.map