import {
  __publicField
} from "../../chunk-F3FYYIAV.js";
import { deepAssignWithOverwrite } from "@homer0/deep-assign";
import { providerCreator } from "../../utils";
class EnsureBearerToken {
  /**
   * @param options  The options to construct the class.
   */
  constructor({
    inject: { HTTPError, statuses },
    ...options
  }) {
    /**
     * To generate the errors when the validation fails.
     */
    __publicField(this, "_HTTPError");
    /**
     * The customization options for the service.
     */
    __publicField(this, "_options");
    this._HTTPError = HTTPError;
    this._options = deepAssignWithOverwrite(
      {
        error: {
          message: "Unauthorized",
          status: statuses("unauthorized"),
          response: {}
        },
        expression: /bearer (.*?)(?:$|\s)/i,
        local: "token"
      },
      options
    );
  }
  /**
   * Generates the middleware that verifies if a request has an `Authorization` header
   * with a bearer token.
   */
  getMiddleware() {
    return (req, res, next) => {
      let unauthorized = true;
      const {
        headers: { authorization }
      } = req;
      if (authorization) {
        const matches = this._options.expression.exec(authorization);
        if (matches) {
          const [, token] = matches;
          res.locals[this._options.local] = token;
          unauthorized = false;
        }
      }
      if (unauthorized) {
        const {
          error: { message, status, response }
        } = this._options;
        next(
          new this._HTTPError(message, status, {
            response
          })
        );
      } else {
        next();
      }
    };
  }
  /**
   * The customization options.
   */
  get options() {
    return deepAssignWithOverwrite({}, this._options);
  }
}
const ensureBearerTokenProvider = providerCreator(
  (options = {}) => (app) => {
    const { serviceName = "ensureBearerToken", ...rest } = options;
    app.set(
      serviceName,
      () => new EnsureBearerToken({
        inject: {
          HTTPError: app.get("HTTPError"),
          statuses: app.get("statuses")
        },
        ...rest
      }).getMiddleware()
    );
  }
);
export {
  EnsureBearerToken,
  ensureBearerTokenProvider
};
//# sourceMappingURL=ensureBearerToken.js.map