"use strict";
var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __defNormalProp = (obj, key, value) => key in obj ? __defProp(obj, key, { enumerable: true, configurable: true, writable: true, value }) : obj[key] = value;
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __copyProps = (to, from, except, desc) => {
  if (from && typeof from === "object" || typeof from === "function") {
    for (let key of __getOwnPropNames(from))
      if (!__hasOwnProp.call(to, key) && key !== except)
        __defProp(to, key, { get: () => from[key], enumerable: !(desc = __getOwnPropDesc(from, key)) || desc.enumerable });
  }
  return to;
};
var __toCommonJS = (mod) => __copyProps(__defProp({}, "__esModule", { value: true }), mod);
var __publicField = (obj, key, value) => {
  __defNormalProp(obj, typeof key !== "symbol" ? key + "" : key, value);
  return value;
};
var errorHandler_exports = {};
__export(errorHandler_exports, {
  ErrorHandler: () => ErrorHandler,
  errorHandlerMiddleware: () => errorHandlerMiddleware
});
module.exports = __toCommonJS(errorHandler_exports);
var import_deep_assign = require("@homer0/deep-assign");
var import_utils = require("../../utils");
var import_services = require("../../services");
class ErrorHandler {
  /**
   * @param options  The options to construct the class.
   */
  constructor({
    inject: { logger, responsesBuilder, statuses, HTTPError },
    ...options
  }) {
    /**
     * The service that will log the messages in the console.
     */
    __publicField(this, "_logger");
    /**
     * The service to generate the responses.
     */
    __publicField(this, "_responsesBuilder");
    /**
     * The uility service to get HTTP status codes.
     */
    __publicField(this, "_statuses");
    /**
     * The Error class used by the "known errors".
     */
    __publicField(this, "_HTTPError");
    /**
     * The customization options.
     */
    __publicField(this, "_options");
    this._logger = logger;
    this._responsesBuilder = responsesBuilder;
    this._statuses = statuses;
    this._HTTPError = HTTPError;
    this._options = (0, import_deep_assign.deepAssignWithOverwrite)(
      {
        showErrors: false,
        response: {
          message: "Unexpected error",
          status: this._statuses("internal server error")
        }
      },
      options
    );
  }
  /**
   * Generates the middleware that handles the errors.
   */
  getMiddleware() {
    return (err, _, res, next) => {
      if (!err) {
        next();
        return;
      }
      const { response, showErrors } = this._options;
      let { status } = response;
      let data = {
        error: true,
        message: response.message
      };
      const knownError = err instanceof import_services.AppError;
      if (showErrors || knownError) {
        data.message = err.message;
        if (knownError) {
          data = {
            ...data,
            ...err.getResponse()
          };
          status = err.status || this._statuses("bad request");
        }
        if (showErrors && err instanceof Error && err.stack) {
          const stack = err.stack.split("\n").map((line) => line.trim());
          data.stack = stack;
          stack.splice(0, 1);
          this._logger.error(`ERROR: ${err.message}`);
          this._logger.info(stack);
        }
      }
      this._responsesBuilder.json({
        res,
        data,
        status
      });
    };
  }
  /**
   * The handler customization options.
   */
  get options() {
    return (0, import_deep_assign.deepAssignWithOverwrite)({}, this._options);
  }
}
const errorHandlerMiddleware = (0, import_utils.middlewareCreator)(
  (options = {}) => (app) => {
    const showErrors = app.getConfig("debug.showErrors") === true;
    return new ErrorHandler({
      inject: {
        logger: app.get("logger"),
        responsesBuilder: app.get("responsesBuilder"),
        statuses: app.get("statuses"),
        HTTPError: app.get("HTTPError")
      },
      showErrors,
      ...options
    }).getMiddleware();
  }
);
// Annotate the CommonJS export names for ESM import in node:
0 && (module.exports = {
  ErrorHandler,
  errorHandlerMiddleware
});
//# sourceMappingURL=errorHandler.js.map