"use strict";
var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __defNormalProp = (obj, key, value) => key in obj ? __defProp(obj, key, { enumerable: true, configurable: true, writable: true, value }) : obj[key] = value;
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __copyProps = (to, from, except, desc) => {
  if (from && typeof from === "object" || typeof from === "function") {
    for (let key of __getOwnPropNames(from))
      if (!__hasOwnProp.call(to, key) && key !== except)
        __defProp(to, key, { get: () => from[key], enumerable: !(desc = __getOwnPropDesc(from, key)) || desc.enumerable });
  }
  return to;
};
var __toCommonJS = (mod) => __copyProps(__defProp({}, "__esModule", { value: true }), mod);
var __publicField = (obj, key, value) => {
  __defNormalProp(obj, typeof key !== "symbol" ? key + "" : key, value);
  return value;
};
var hsts_exports = {};
__export(hsts_exports, {
  HSTS: () => HSTS,
  hstsMiddleware: () => hstsMiddleware
});
module.exports = __toCommonJS(hsts_exports);
var import_utils = require("../../utils");
class HSTS {
  /**
   * @param options  The options to construct the class.
   */
  constructor(options = {}) {
    /**
     * The customization options for the header.
     */
    __publicField(this, "_options");
    /**
     * The value of the header that will be included in the responses.
     */
    __publicField(this, "_header");
    this._options = {
      maxAge: 31536e3,
      includeSubDomains: true,
      preload: false,
      ...options
    };
    this._header = this._buildHeader();
  }
  /**
   * Generates the middleware that includes the HSTS header on the responses.
   */
  getMiddleware() {
    return (_, res, next) => {
      res.setHeader("Strict-Transport-Security", this._header);
      next();
    };
  }
  /**
   * The customization options.
   */
  get options() {
    return { ...this._options };
  }
  /**
   * The value of the header that will be included in the responses.
   */
  get header() {
    return this._header;
  }
  /**
   * Creates the header value based on the customization options.
   */
  _buildHeader() {
    const { maxAge, includeSubDomains, preload } = this._options;
    const directives = [`max-age=${maxAge}`];
    if (includeSubDomains) {
      directives.push("includeSubDomains");
    }
    if (preload) {
      directives.push("preload");
    }
    return directives.join("; ");
  }
}
const hstsMiddleware = (0, import_utils.middlewareCreator)(
  (options = {}) => (app) => {
    const setting = app.getConfig("hsts");
    if (!setting?.enabled || typeof options.enabled !== "undefined" && !options.enabled)
      return void 0;
    return new HSTS(options).getMiddleware();
  }
);
// Annotate the CommonJS export names for ESM import in node:
0 && (module.exports = {
  HSTS,
  hstsMiddleware
});
//# sourceMappingURL=hsts.js.map