"use strict";
var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __defNormalProp = (obj, key, value) => key in obj ? __defProp(obj, key, { enumerable: true, configurable: true, writable: true, value }) : obj[key] = value;
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __copyProps = (to, from, except, desc) => {
  if (from && typeof from === "object" || typeof from === "function") {
    for (let key of __getOwnPropNames(from))
      if (!__hasOwnProp.call(to, key) && key !== except)
        __defProp(to, key, { get: () => from[key], enumerable: !(desc = __getOwnPropDesc(from, key)) || desc.enumerable });
  }
  return to;
};
var __toCommonJS = (mod) => __copyProps(__defProp({}, "__esModule", { value: true }), mod);
var __publicField = (obj, key, value) => {
  __defNormalProp(obj, typeof key !== "symbol" ? key + "" : key, value);
  return value;
};
var showHTML_exports = {};
__export(showHTML_exports, {
  ShowHTML: () => ShowHTML,
  showHTMLMiddleware: () => showHTMLMiddleware
});
module.exports = __toCommonJS(showHTML_exports);
var import_utils = require("../../utils");
class ShowHTML {
  /**
   * @param options  The options to construct the class.
   */
  constructor({ inject, ...options }) {
    /**
     * The service that serves a file.
     */
    __publicField(this, "_sendFile");
    /**
     * A function to get a possible {@link HTMLGenerator}. This is injected as a "getter"
     * to not interrupt the DIC "lifecycle": middlewares are initialized right when the
     * app starts, and injecting a reference would force the service to be initialized
     * too, even if a request is not being made.
     */
    __publicField(this, "_getHTMLGenerator");
    /**
     * The customization options for the middleware.
     */
    __publicField(this, "_options");
    /**
     * Whether or not the file is ready to be served. In case the middleware uses an
     * {@link HTMLGenerator} service, the file needs to be generated before being available,
     * and that's why this flag exists.
     */
    __publicField(this, "_fileReady", false);
    this._sendFile = inject.sendFile;
    this._getHTMLGenerator = inject.getHTMLGenerator || (() => void 0);
    this._options = {
      file: "index.html",
      ...options
    };
  }
  /**
   * Generates the middleware that serves the HTML file.
   */
  getMiddleware() {
    return async (_, res, next) => {
      if (this._fileReady) {
        this._sendResponse(res, next);
        return;
      }
      const htmlGenerator = this._getHTMLGenerator();
      if (!htmlGenerator) {
        this._fileReady = true;
        this._sendResponse(res, next);
        return;
      }
      try {
        await htmlGenerator.whenReady();
        this._options.file = htmlGenerator.options.file;
        this._fileReady = true;
        this._sendResponse(res, next);
      } catch (error) {
        next(error);
      }
    };
  }
  /**
   * The customization options.
   */
  get options() {
    return { ...this._options };
  }
  /**
   * Serves the HTML file to the response.
   *
   * @param res   The response object generated by the application.
   * @param next  The function to call the next middleware.
   */
  _sendResponse(res, next) {
    res.setHeader("Content-Type", "text/html");
    this._sendFile({
      res,
      next,
      filepath: this._options.file
    });
  }
}
const showHTMLMiddleware = (0, import_utils.middlewareCreator)(
  (options = {}) => (app) => {
    const { htmlGeneratorServiceName = "htmlGenerator", ...rest } = options;
    return new ShowHTML({
      inject: {
        sendFile: app.get("sendFile"),
        getHTMLGenerator: () => app.try(htmlGeneratorServiceName)
      },
      ...rest
    }).getMiddleware();
  }
);
// Annotate the CommonJS export names for ESM import in node:
0 && (module.exports = {
  ShowHTML,
  showHTMLMiddleware
});
//# sourceMappingURL=showHTML.js.map