"use strict";
var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __defNormalProp = (obj, key, value) => key in obj ? __defProp(obj, key, { enumerable: true, configurable: true, writable: true, value }) : obj[key] = value;
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __copyProps = (to, from, except, desc) => {
  if (from && typeof from === "object" || typeof from === "function") {
    for (let key of __getOwnPropNames(from))
      if (!__hasOwnProp.call(to, key) && key !== except)
        __defProp(to, key, { get: () => from[key], enumerable: !(desc = __getOwnPropDesc(from, key)) || desc.enumerable });
  }
  return to;
};
var __toCommonJS = (mod) => __copyProps(__defProp({}, "__esModule", { value: true }), mod);
var __publicField = (obj, key, value) => {
  __defNormalProp(obj, typeof key !== "symbol" ? key + "" : key, value);
  return value;
};
var versionValidator_exports = {};
__export(versionValidator_exports, {
  VersionValidator: () => VersionValidator,
  versionValidatorMiddleware: () => versionValidatorMiddleware
});
module.exports = __toCommonJS(versionValidator_exports);
var import_deep_assign = require("@homer0/deep-assign");
var import_utils = require("../../utils");
class VersionValidator {
  /**
   * @param options  The options to construct the class.
   * @throws If no `version` is specified in the options.
   */
  constructor({ inject, version, ...options }) {
    /**
     * To generate the errors in case the validation fails.
     */
    __publicField(this, "_HTTPError");
    /**
     * To generate responses for popups.
     */
    __publicField(this, "_responsesBuilder");
    /**
     * The uility service to get HTTP status codes.
     */
    __publicField(this, "_statuses");
    /**
     * The customization options.
     */
    __publicField(this, "_options");
    this._HTTPError = inject.HTTPError;
    this._responsesBuilder = inject.responsesBuilder;
    this._statuses = inject.statuses;
    this._options = (0, import_deep_assign.deepAssignWithOverwrite)(
      {
        error: "The application version doesn't match",
        latest: {
          allow: true,
          name: "latest"
        },
        popup: {
          variable: "popup",
          title: "Conflict",
          message: "vesion:conflict"
        },
        version
      },
      options
    );
    if (!this._options.version) {
      throw new Error("You need to supply a version");
    }
  }
  /**
   * Generates the middleware that validates the version.
   */
  getMiddleware() {
    return (req, res, next) => {
      const { version } = req.params;
      if (!version) {
        next();
        return;
      }
      if (version === this._options.version || this._isTheAllowedLatest(version)) {
        next();
        return;
      }
      const status = this._statuses("conflict");
      if (this._isPopup(req)) {
        const { title, message } = this._options.popup;
        this._responsesBuilder.htmlPostMessage({
          res,
          title,
          message,
          status
        });
        return;
      }
      next(
        new this._HTTPError(this._options.error, status, {
          response: {
            validation: true
          }
        })
      );
    };
  }
  /**
   * The customization options.
   */
  get options() {
    return (0, import_deep_assign.deepAssignWithOverwrite)({}, this._options);
  }
  /**
   * Helper method that checks if the incoming request is from a popup. It will look for
   * the query string variable defined in the constructor options.
   *
   * @param req  The request object sent by the application.
   */
  _isPopup(req) {
    const popup = req.query[this._options.popup.variable];
    return !!(popup && String(popup).toLowerCase() === "true");
  }
  /**
   * Helper method that checks if the "latest version" is enabled and if the given version
   * is "the latest" (comparing it with the option name).
   *
   * @param version  The version received in the request.
   */
  _isTheAllowedLatest(version) {
    const { allow, name } = this._options.latest;
    return allow && version === name;
  }
}
const versionValidatorMiddleware = (0, import_utils.middlewareCreator)(
  (options = {}) => (app, route) => {
    const version = app.getConfig("version");
    const middleware = new VersionValidator({
      inject: {
        HTTPError: app.get("HTTPError"),
        responsesBuilder: app.get("responsesBuilder"),
        statuses: app.get("statuses")
      },
      version,
      ...options
    }).getMiddleware();
    if (route) {
      const router = app.getRouter();
      return router.all("/:version/*", middleware);
    }
    return middleware;
  }
);
// Annotate the CommonJS export names for ESM import in node:
0 && (module.exports = {
  VersionValidator,
  versionValidatorMiddleware
});
//# sourceMappingURL=versionValidator.js.map