"use strict";
var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __defNormalProp = (obj, key, value) => key in obj ? __defProp(obj, key, { enumerable: true, configurable: true, writable: true, value }) : obj[key] = value;
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __copyProps = (to, from, except, desc) => {
  if (from && typeof from === "object" || typeof from === "function") {
    for (let key of __getOwnPropNames(from))
      if (!__hasOwnProp.call(to, key) && key !== except)
        __defProp(to, key, { get: () => from[key], enumerable: !(desc = __getOwnPropDesc(from, key)) || desc.enumerable });
  }
  return to;
};
var __toCommonJS = (mod) => __copyProps(__defProp({}, "__esModule", { value: true }), mod);
var __publicField = (obj, key, value) => {
  __defNormalProp(obj, typeof key !== "symbol" ? key + "" : key, value);
  return value;
};
var appError_exports = {};
__export(appError_exports, {
  AppError: () => AppError,
  appErrorProvider: () => appErrorProvider,
  createAppError: () => createAppError
});
module.exports = __toCommonJS(appError_exports);
var import_utils = require("../../utils");
class AppError extends Error {
  /**
   * @param message   The message of the error.
   * @param context   The context information, for the error handler.
   * @param statuses  A reference to the service that generates HTTP status codes. This
   *                  is in case the implementation wants to use a special version from
   *                  the container; otherwise, it will use the `statuses` library
   *                  directly.
   */
  constructor(message, context = {}, statuses = import_utils.statuses) {
    super(message);
    /**
     * The date of when the error was generated.
     */
    __publicField(this, "_date");
    /**
     * The context information that can be provided to the error, and picked by the error
     * handler.
     */
    __publicField(this, "_context");
    /**
     * The service that generates HTTP status codes.
     */
    __publicField(this, "_statuses");
    this.name = this.constructor.name;
    this._date = /* @__PURE__ */ new Date();
    this._statuses = statuses;
    this._context = this._parseContext(context);
    if (Error.captureStackTrace) {
      Error.captureStackTrace(this, this.constructor);
    }
  }
  /**
   * Gets an object that can be included in a response from the application. This method
   * is a helper for the `response` getter, as it allows for the assertion of the response
   * type.
   *
   * @template T  The type of the response.
   */
  getResponse() {
    return this.response;
  }
  /**
   * Information about the error that can be included in a response. This is set using the
   * `response` key in the `context` option.
   */
  get response() {
    return this._context.response || {};
  }
  /**
   * An HTTP status code related to the error. This is set using the `status` key on the
   * `context`.
   */
  get status() {
    return this._context.status;
  }
  /**
   * Context information related to the error.
   */
  get context() {
    return this._context;
  }
  /**
   * The date of when the error was generated.
   */
  get date() {
    return this._date;
  }
  /**
   * Utility method that formats the context before saving it in the instance:
   * - If the context includes a `status` as a `string`, it will try to replace it with
   * its actual status code from the `statuses `service.
   *
   * @param context  The original context sent to the constructor.
   */
  _parseContext(context) {
    const result = { ...context };
    if (result.status && typeof result.status === "string") {
      result.status = this._statuses.code[result.status.toLowerCase()] || result.status;
    }
    return result;
  }
}
const createAppError = (...args) => new AppError(...args);
const appErrorProvider = (0, import_utils.provider)((app) => {
  app.set("AppError", () => AppError);
  app.set("appError", () => createAppError);
});
// Annotate the CommonJS export names for ESM import in node:
0 && (module.exports = {
  AppError,
  appErrorProvider,
  createAppError
});
//# sourceMappingURL=appError.js.map