"use strict";
var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __defNormalProp = (obj, key, value) => key in obj ? __defProp(obj, key, { enumerable: true, configurable: true, writable: true, value }) : obj[key] = value;
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __copyProps = (to, from, except, desc) => {
  if (from && typeof from === "object" || typeof from === "function") {
    for (let key of __getOwnPropNames(from))
      if (!__hasOwnProp.call(to, key) && key !== except)
        __defProp(to, key, { get: () => from[key], enumerable: !(desc = __getOwnPropDesc(from, key)) || desc.enumerable });
  }
  return to;
};
var __toCommonJS = (mod) => __copyProps(__defProp({}, "__esModule", { value: true }), mod);
var __publicField = (obj, key, value) => {
  __defNormalProp(obj, typeof key !== "symbol" ? key + "" : key, value);
  return value;
};
var htmlGenerator_exports = {};
__export(htmlGenerator_exports, {
  HTMLGenerator: () => HTMLGenerator,
  htmlGeneratorProvider: () => htmlGeneratorProvider
});
module.exports = __toCommonJS(htmlGenerator_exports);
var import_object_utils = require("@homer0/object-utils");
var import_deep_assign = require("@homer0/deep-assign");
var import_deferred = require("@homer0/deferred");
var import_utils = require("../../utils");
class HTMLGenerator {
  /**
   * @param options  The options to construct the class.
   * @throws If `valuesService` is specified but it doesn't have a `getValues`
   *         method.
   */
  constructor({
    inject: { config, logger, frontendFs, valuesService },
    ...options
  }) {
    /**
     * The service customization options.
     */
    __publicField(this, "_options");
    /**
     * The application configuration service, to get the settings specified by the
     * `configKeys` option.
     */
    __publicField(this, "_config");
    /**
     * The service that logs messages on the terminal, in case the `silent` option is `false`.
     */
    __publicField(this, "_logger");
    /**
     * The service that interacts with the filesystem.
     */
    __publicField(this, "_frontendFs");
    /**
     * A service that can provide values to be replaced in the template.
     */
    __publicField(this, "_valuesService");
    /**
     * Whether or not the file was already generated.
     */
    __publicField(this, "_fileReady", false);
    /**
     * A deferred promise to return when another service asks if the file has been
     * generated. Once this sevice finishes generating the file, the promise will be
     * resolved for all implementations that hold a reference to this deferred.
     */
    __publicField(this, "_fileDeferred");
    this._config = config;
    this._logger = logger;
    this._frontendFs = frontendFs;
    this._valuesService = valuesService;
    this._options = (0, import_deep_assign.deepAssignWithOverwrite)(
      {
        template: "index.tpl.html",
        file: "index.html",
        silent: false,
        deleteTemplateAfter: true,
        replacePlaceholder: /\{\{appConfig(?:uration)?\}\}/,
        placeholderExpression: /\{\{(.*?)\}\}/gi,
        variableName: "appConfig",
        configKeys: ["features", "version", "postMessagesPrefix"]
      },
      options
    );
    if (this._valuesService && typeof this._valuesService.getValues !== "function") {
      throw new Error("The HTMLGenerator values service must have a `getValues` method");
    }
  }
  /**
   * Gets a promise that will be resolved when the file has been generated.
   */
  whenReady() {
    return this._fileDeferred ? this._fileDeferred.promise : Promise.resolve();
  }
  /**
   * Generates the HTML file.
   */
  async generateHTML() {
    if (this._fileReady)
      return void 0;
    if (this._fileDeferred)
      return this._fileDeferred.promise;
    this._fileDeferred = (0, import_deferred.deferred)();
    const { template, deleteTemplateAfter, file, silent } = this._options;
    try {
      const templateContents = await this._frontendFs.read(template);
      const values = await this._getValues();
      const html = this._processHTML(templateContents, values);
      await this._frontendFs.write(file, html);
      if (!silent) {
        this._logger.success(`The HTML file was successfully generated (${file})`);
      }
      if (deleteTemplateAfter) {
        await this._frontendFs.delete(template);
        if (!silent) {
          this._logger.info(`The HTML template was successfully removed (${template})`);
        }
      }
      this._fileReady = true;
      this._fileDeferred.resolve();
      this._fileDeferred = void 0;
      return void 0;
    } catch (error) {
      this._fileDeferred.reject(error);
      this._fileDeferred = void 0;
      if (!silent) {
        this._logger.error("There was an error while generating the HTML");
      }
      throw error;
    }
  }
  /**
   * Gets the customization options.
   */
  get options() {
    return (0, import_deep_assign.deepAssignWithOverwrite)({}, this._options);
  }
  /**
   * Helper method to get the values that will be replaced in the template. If a "values
   * service" was specified in the constructor, it will get the values from there,
   * otherwise, it will use the `configKeys` option to get the values from the
   * application configuration.
   */
  _getValues() {
    if (this._valuesService) {
      return this._valuesService.getValues(this._options);
    }
    const { configKeys } = this._options;
    if (configKeys && configKeys.length) {
      return Promise.resolve(this._config.get(configKeys));
    }
    return Promise.resolve({});
  }
  /**
   * Processes the HTML template by replacing the placeholders with the actual values.
   *
   * @param template  The template for the HTML file.
   * @param values    The values dictionary that should be replaced in the template.
   */
  _processHTML(template, values) {
    const { replacePlaceholder, placeholderExpression, variableName } = this._options;
    const htmlObject = JSON.stringify(values);
    let code = template.replace(
      replacePlaceholder,
      `window.${variableName} = ${htmlObject}`
    );
    const matches = [];
    let match = placeholderExpression.exec(code);
    while (match) {
      const [string, value] = match;
      if (string && value) {
        matches.push({
          string,
          value
        });
      }
      match = placeholderExpression.exec(code);
    }
    matches.forEach((info) => {
      code = code.replace(info.string, String((0, import_object_utils.get)(values, info.value)));
    });
    return code;
  }
}
const htmlGeneratorProvider = (0, import_utils.providerCreator)(
  (options = {}) => (app) => {
    const {
      serviceName = "htmlGenerator",
      valuesServiceName = "htmlGeneratorValues",
      ...rest
    } = options;
    app.set(
      serviceName,
      () => new HTMLGenerator({
        inject: {
          config: app.get("config"),
          logger: app.get("logger"),
          frontendFs: app.get("frontendFs"),
          valuesService: app.try(valuesServiceName)
        },
        ...rest
      })
    );
    app.once("afterStart", () => {
      app.get(serviceName).generateHTML();
    });
  }
);
// Annotate the CommonJS export names for ESM import in node:
0 && (module.exports = {
  HTMLGenerator,
  htmlGeneratorProvider
});
//# sourceMappingURL=htmlGenerator.js.map