"use strict";
var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __defNormalProp = (obj, key, value) => key in obj ? __defProp(obj, key, { enumerable: true, configurable: true, writable: true, value }) : obj[key] = value;
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __copyProps = (to, from, except, desc) => {
  if (from && typeof from === "object" || typeof from === "function") {
    for (let key of __getOwnPropNames(from))
      if (!__hasOwnProp.call(to, key) && key !== except)
        __defProp(to, key, { get: () => from[key], enumerable: !(desc = __getOwnPropDesc(from, key)) || desc.enumerable });
  }
  return to;
};
var __toCommonJS = (mod) => __copyProps(__defProp({}, "__esModule", { value: true }), mod);
var __publicField = (obj, key, value) => {
  __defNormalProp(obj, typeof key !== "symbol" ? key + "" : key, value);
  return value;
};
var ensureBearerToken_exports = {};
__export(ensureBearerToken_exports, {
  EnsureBearerToken: () => EnsureBearerToken,
  ensureBearerTokenProvider: () => ensureBearerTokenProvider
});
module.exports = __toCommonJS(ensureBearerToken_exports);
var import_deep_assign = require("@homer0/deep-assign");
var import_utils = require("../../utils");
class EnsureBearerToken {
  /**
   * @param options  The options to construct the class.
   */
  constructor({
    inject: { HTTPError, statuses },
    ...options
  }) {
    /**
     * To generate the errors when the validation fails.
     */
    __publicField(this, "_HTTPError");
    /**
     * The customization options for the service.
     */
    __publicField(this, "_options");
    this._HTTPError = HTTPError;
    this._options = (0, import_deep_assign.deepAssignWithOverwrite)(
      {
        error: {
          message: "Unauthorized",
          status: statuses("unauthorized"),
          response: {}
        },
        expression: /bearer (.*?)(?:$|\s)/i,
        local: "token"
      },
      options
    );
  }
  /**
   * Generates the middleware that verifies if a request has an `Authorization` header
   * with a bearer token.
   */
  getMiddleware() {
    return (req, res, next) => {
      let unauthorized = true;
      const {
        headers: { authorization }
      } = req;
      if (authorization) {
        const matches = this._options.expression.exec(authorization);
        if (matches) {
          const [, token] = matches;
          res.locals[this._options.local] = token;
          unauthorized = false;
        }
      }
      if (unauthorized) {
        const {
          error: { message, status, response }
        } = this._options;
        next(
          new this._HTTPError(message, status, {
            response
          })
        );
      } else {
        next();
      }
    };
  }
  /**
   * The customization options.
   */
  get options() {
    return (0, import_deep_assign.deepAssignWithOverwrite)({}, this._options);
  }
}
const ensureBearerTokenProvider = (0, import_utils.providerCreator)(
  (options = {}) => (app) => {
    const { serviceName = "ensureBearerToken", ...rest } = options;
    app.set(
      serviceName,
      () => new EnsureBearerToken({
        inject: {
          HTTPError: app.get("HTTPError"),
          statuses: app.get("statuses")
        },
        ...rest
      }).getMiddleware()
    );
  }
);
// Annotate the CommonJS export names for ESM import in node:
0 && (module.exports = {
  EnsureBearerToken,
  ensureBearerTokenProvider
});
//# sourceMappingURL=ensureBearerToken.js.map