import { Server } from 'https';
export { Server as HTTPSServer } from 'https';
import { Server as Server$1 } from 'http';
export { Server as HTTPServer } from 'http';
import { ServerOptions } from 'spdy';
import NodeFetchFn from 'node-fetch';
export { Response as HTTPResponse } from 'node-fetch';
import { Express } from 'express';

type NodeFetch = typeof NodeFetchFn;

/**
 * @group Jimpex
 */
type JimpexServer = Express | Server;
/**
 * @group Jimpex
 */
type JimpexServerInstance = Server$1 | Server;
/**
 * The paths to the SSL credentials. Depending on the application options, they can be
 * relative to the project root, or the application executable, but they can't be
 * absolute.
 *
 * @group Jimpex
 */
type JimpexHTTPSCredentials = {
    /**
     * The path to the certificate authority file.
     */
    ca?: string;
    /**
     * The path to the certificate file.
     */
    cert?: string;
    /**
     * The path to the key file.
     */
    key?: string;
};
/**
 * The options to enable HTTP2.
 *
 * @group Jimpex
 */
type JimpexHTTP2Options = {
    /**
     * Whether or not to enable HTTP2.
     */
    enabled?: boolean;
    /**
     * Custom options for the Spdy server.
     *
     * @see {@link https://github.com/spdy-http2/node-spdy#options}
     */
    spdy?: ServerOptions['spdy'];
};
/**
 * The options to enable HTTPS.
 *
 * @group Jimpex
 */
type JimpexHTTPSOptions = {
    /**
     * Whether or not to enable HTTPS.
     */
    enabled?: boolean;
    /**
     * The SSL credentials, and their location.
     */
    credentials?: JimpexHTTPSCredentials & {
        /**
         * If `true`, the credentials will be located relative to the project root, otherwise,
         * they will be located relative to the application executable.
         *
         * @default false
         */
        onHome?: boolean;
    };
};

export type { JimpexHTTP2Options, JimpexHTTPSCredentials, JimpexHTTPSOptions, JimpexServer, JimpexServerInstance, NodeFetch };
